"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Moderator = void 0;
const enmap_1 = __importDefault(require("enmap"));
const discord_js_1 = require("discord.js");
const ModeratorEmitter_1 = require("./ModeratorEmitter");
const MuteManager_1 = require("./managers/MuteManager");
const WarnManager_1 = require("./managers/WarnManager");
const FetchManager_1 = require("./managers/FetchManager");
const UtilsManager_1 = require("./managers/UtilsManager");
const ModeratorEnums_1 = require("./ModeratorEnums");
const package_json_1 = require("../package.json");
/**
 * Class that controls Moderation System
 *
 * @class
 * @classdesc Moderator Class
 * @extends {ModeratorEmitter}
 */
class Moderator extends ModeratorEmitter_1.ModeratorEmitter {
    client;
    database;
    author;
    website;
    version;
    ready;
    readyTimestamp;
    options;
    mutes;
    warns;
    fetcher;
    utils;
    /**
     * @constructor
     *
     * @param {Client} client Discord Client
     * @param {ModeratorOptions} [options] Module Options
     */
    constructor(client, options) {
        super();
        /**
         * Discord Client
         *
         * @type {Client}
         * @private
         */
        this.client = client;
        /**
         * Moderator Database
         *
         * @type {?Enmap}
         */
        this.database = null;
        /**
         * Moderator Author
         *
         * @type {string}
         */
        this.author = package_json_1.author;
        /**
         * Moderator WebSite URL
         *
         * @type {string}
         */
        this.website = package_json_1.homepage;
        /**
         * Moderator Version
         *
         * @type {string}
         */
        this.version = package_json_1.version;
        /**
         * Moderator Ready Status
         *
         * @type {?boolean}
         */
        this.ready = null;
        /**
         * Moderator Options
         *
         * @type {?ModeratorOptions}
         */
        this.options = null;
        /**
         * Moderator Mute Manager
         *
         * @type {?MuteManager}
         */
        this.mutes = null;
        /**
         * Moderator Warn Manager
         *
         * @type {?WarnManager}
         */
        this.warns = null;
        /**
         * Moderator Fetch Manager
         *
         * @type {?FetchManager}
         */
        this.fetcher = null;
        /**
         * Moderator Utils Manager
         *
         * @type {UtilsManager}
         */
        this.utils = new UtilsManager_1.UtilsManager(this);
        this.init(options);
    }
    /**
     * Method for kicking Discord guild users
     *
     * @param {KickOptions} options Kick options
     *
     * @returns {Promise<DefaultObject>} Returns the state of the action or an object with an error
     */
    kick(options) {
        return new Promise(async (res, rej) => {
            let guild = null;
            let member = null;
            if (options.guild instanceof discord_js_1.Guild)
                guild = options.guild;
            else
                guild = await this.fetcher.guild(options.guild);
            if (!guild)
                return res({
                    status: false,
                    message: `Discord guild with ID '${options.guild}' is not found in the client!`
                });
            if (options.user instanceof discord_js_1.GuildMember)
                member = options.user;
            else
                member = await this.fetcher.member(guild.id, options.user);
            if (!member)
                return res({
                    status: false,
                    message: `The user with ID '${options.user instanceof discord_js_1.GuildMember ? options.user.id : options.user}' is not found in the Discord guild with ID '${options.guild instanceof discord_js_1.Guild ? options.guild.id : options.guild}'!`
                });
            if (!member.kickable)
                return res({
                    status: false,
                    message: `A target with ID '${member.id}' cannot be kicked!`
                });
            try {
                await member.kick(options.reason);
            }
            catch (error) {
                return res({
                    status: false,
                    message: error.message
                });
            }
            this.emit(ModeratorEnums_1.Events.KICK, member.guild, member);
            return res({ status: true });
        });
    }
    /**
     * Method for banning Discord guild users
     *
     * @param {BanOptions} options Ban options
     *
     * @returns {Promise<DefaultObject>} Returns the state of the action or an object with an error
     */
    ban(options) {
        return new Promise(async (res, rej) => {
            let guild = null;
            let member = null;
            if (options.guild instanceof discord_js_1.Guild)
                guild = options.guild;
            else
                guild = await this.fetcher.guild(options.guild);
            if (!guild)
                return res({
                    status: false,
                    message: `Discord guild with ID '${options.guild}' is not found in the client!`
                });
            if (options.user instanceof discord_js_1.GuildMember)
                member = options.user;
            else
                member = await this.fetcher.member(guild.id, options.user);
            if (!member)
                return res({
                    status: false,
                    message: `The user with ID '${options.user instanceof discord_js_1.GuildMember ? options.user.id : options.user}' is not found in the Discord guild with ID '${options.guild instanceof discord_js_1.Guild ? options.guild.id : options.guild}'!`
                });
            if (!member.bannable)
                return res({
                    status: false,
                    message: `The target with ID '${member.id}' cannot be banned!`
                });
            try {
                await member.ban({ reason: options.reason });
            }
            catch (error) {
                return res({
                    status: false,
                    message: error.message
                });
            }
            this.emit(ModeratorEnums_1.Events.BAN, member.guild, member);
            return res({ status: true });
        });
    }
    /**
     * Method for removing bans from Discord guild users
     *
     * @param {UnbanOptions} options Unban options
     *
     * @returns {Promise<DefaultObject>} Returns the state of the action or an object with an error
     */
    unban(options) {
        return new Promise(async (res, rej) => {
            let guild = null;
            if (options.guild instanceof discord_js_1.Guild)
                guild = options.guild;
            else
                guild = await this.fetcher.guild(options.guild);
            if (!guild)
                return res({
                    status: false,
                    message: `Discord guild with ID '${options.guild}' is not found in the client!`
                });
            const guildBans = await guild.bans.fetch();
            if (!guildBans || !guildBans?.size)
                return res({
                    status: false,
                    message: `Bans not found in Discord guild with ID '${options.guild}'!`
                });
            const guildMemberBan = guildBans.find(guildBan => guildBan.user.id === options.user);
            if (!guildMemberBan)
                return res({
                    status: false,
                    message: `The target with ID '${options.user}' has no bans on the server with ID '${options.guild}'!`
                });
            try {
                await guild.members?.unban(guildMemberBan.user);
            }
            catch (error) {
                return res({
                    status: false,
                    message: error.message
                });
            }
            this.emit(ModeratorEnums_1.Events.UNBAN, guildMemberBan.guild, guildMemberBan.user);
            return res({ status: true });
        });
    }
    /**
     * Method for punishing Discord guild users
     *
     * @param {PunishOptions} options Punish options
     *
     * @returns {Promise<DefaultObject>} Returns the state of the action or an object with an error
     */
    punish(options) {
        return new Promise(async (res, rej) => {
            let isSuccess = false;
            let guild = null;
            let member = null;
            if (options.guild instanceof discord_js_1.Guild)
                guild = options.guild;
            else
                guild = await this.fetcher.guild(options.guild);
            if (!guild)
                return res({
                    status: false,
                    message: `Discord guild with ID '${options.guild}' is not found in the client!`
                });
            if (options.user instanceof discord_js_1.GuildMember)
                member = options.user;
            else
                member = await this.fetcher.member(guild.id, options.user);
            switch (options.type) {
                case ModeratorEnums_1.PunishmentType.BAN: {
                    await this.ban({
                        user: member,
                        guild: guild,
                        reason: options.reason
                    });
                    isSuccess = true;
                    break;
                }
                case ModeratorEnums_1.PunishmentType.KICK: {
                    await this.kick({
                        user: member,
                        guild: member.guild.id,
                        reason: options.reason
                    });
                    isSuccess = true;
                    break;
                }
                case ModeratorEnums_1.PunishmentType.MUTE: {
                    await this.mutes?.create({
                        type: ModeratorEnums_1.MuteType.PERMANENT,
                        reason: options.reason,
                        moderator: options.moderator,
                        user: member,
                        guild: guild,
                        channel: options.channel
                    });
                    isSuccess = true;
                    break;
                }
                case ModeratorEnums_1.PunishmentType.TEMPMUTE: {
                    await this.mutes?.create({
                        type: ModeratorEnums_1.MuteType.TEMPORARY,
                        time: options.time,
                        reason: options.reason,
                        moderator: options.moderator,
                        user: member,
                        guild: guild,
                        channel: options.channel
                    });
                    isSuccess = true;
                    break;
                }
            }
            return res({ status: isSuccess });
        });
    }
    /**
     * Method for deleting messages in text channels
     *
     * @param {ClearOptions} options Clear options
     *
     * @returns {Promise<ClearData | DefaultObject>} Returns the number of deleted messages or an object with an error
     */
    clear(options) {
        return new Promise(async (res, rej) => {
            if (new Object(options).hasOwnProperty("count"))
                options.count = 100;
            if (options.count < 1 || options.count > 100)
                return res({
                    status: false,
                    message: `The specified number of '${options.count}' messages violates the Discord limit of 0 to 100 messages!`
                });
            let channel = null;
            if (options.channel instanceof discord_js_1.TextChannel)
                channel = options.channel;
            else
                channel = await this.fetcher.channel(options.channel);
            try {
                const deletedMessages = await channel.bulkDelete(options.count, true);
                return res({
                    count: options.count,
                    deleted: deletedMessages.size
                });
            }
            catch (error) {
                return res({
                    status: false,
                    message: error.message
                });
            }
        });
    }
    /**
     * Method for module initialization
     *
     * @param {ModeratorOptions} [options={}] Moderator Options
     *
     * @private
     */
    init(options) {
        const interval = setInterval(async () => {
            if (this.client.isReady()) {
                this.utils.fetchIntents();
                this.ready = true;
                this.readyTimestamp = Date.now();
                this.options = this.utils.fetchOptions(options ?? {});
                this.initDatabase();
                this.initManagers();
                this.emit(ModeratorEnums_1.Events.READY, this);
                clearInterval(interval);
            }
        }, 100);
    }
    /**
     * Method for initializing the module database
     *
     * @private
     */
    initDatabase() {
        this.database = new enmap_1.default({
            name: this.options.database.name,
            dataDir: this.options.database.path,
            wal: false
        });
    }
    /**
     * Method for initializing all module managers
     *
     * @private
     */
    initManagers() {
        if (this.options.managers.mute.isEnabled)
            this.mutes = new MuteManager_1.MuteManager(this);
        if (this.options.managers.warn.isEnabled)
            this.warns = new WarnManager_1.WarnManager(this);
        this.fetcher = new FetchManager_1.FetchManager(this);
    }
}
exports.Moderator = Moderator;
/**
 * @typedef {object} ModeratorOptions
 *
 * @prop {DatabaseConfig} [database] Moderator Database Configuration
 * @prop {ModeratorManagers} [managers] Moderator Managers Configuration
 */
/**
 * @typedef {object} DatabaseConfig
 *
 * @prop {string} [path="./"] Database Path
 * @prop {string} [name="moderator"] Database Name
 */
/**
 * @typedef {object} ModeratorManagers
 *
 * @prop {MuteManagerConfig} [mute] Moderator Mute Manager
 * @prop {WarnManagerConfig} [warn] Moderator Warn Manager
 */
/**
 * @typedef {object} MuteManagerConfig
 *
 * @prop {boolean} [isEnabled=true] Mute Manager State
 * @prop {number} [interval=5000] Mute Manager Check Interval
 */
/**
 * @typedef {object} WarnManagerConfig
 *
 * @prop {boolean} [isEnabled=true] Warn Manager State
 * @prop {number | [number, number]} [limits=[3, 6]] Warn Manager Limits
 * @prop {WarnPunishmentConfig} [punishment] Warn Manager Punishment
 */
/**
 * @typedef {object} WarnPunishmentConfig
 *
 * @prop {boolean} [isEnabled=true] Warn Manager Punishments State
 * @prop {number} [time=86400] Warn Manager Mute Time
 * @prop {PunishmentType | [PunishmentType, PunishmentType]} [type] Warn Manager Punishments Type
 */
/**
 * @typedef {object} ModeratorGuildData
 *
 * @prop {string} guildID Discord Guild ID
 * @prop {string} muteRole Discord Guild Mute Role ID
 * @prop {Array<GuildMute>} mutes Discord Guild Mutes Array
 * @prop {Array<GuildWarn>} warns Discord Guild Warns Array
 */
/**
 * @typedef {object} GuildMute
 *
 * @prop {string} guildID Discord Guild ID
 * @prop {string} channelID Discord Guild Channel ID
 * @prop {ModeratorUsers} users Mute Users
 * @prop {number} time Mute Time
 * @prop {string} reason Mute Reason
 * @prop {number} mutedAt Date and time of mute in milliseconds
 * @prop {number} unmutedAt Date and time of mute removal in milliseconds
 */
/**
 * @typedef {object} ModeratorUsers
 *
 * @prop {string} target Mute Target ID
 * @prop {string} moderator Mute Moderator ID
 */
/**
 * @typedef {object} DefaultObject
 *
 * @prop {boolean} status Status of a specific operation
 * @prop {string} [message] Text of the error, if any
 */
/**
 * @typedef {object} CreationMuteOptions
 *
 * @prop {number} [time] Mute Time
 * @prop {string} [reason] Mute Reason
 * @prop {string} [moderator] Mute Moderator ID
 * @prop {GuildMember} member Mute Target
 * @prop {TextChannelResolvable} channel Guild Text Channel Resolvable
 */ 
