"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MuteManager = void 0;
const discord_js_1 = require("discord.js");
const ModeratorEnums_1 = require("../ModeratorEnums");
/**
 * Class that controls Moderation Mute System
 *
 * @class
 * @classdesc Moderator Mute Manager Class
 */
class MuteManager {
    moderator;
    /**
     * @constructor
     *
     * @param {Moderator} moderator Moderator Class
     */
    constructor(moderator) {
        /**
         * Moderator Class
         *
         * @type {Moderator}
         * @private
         */
        this.moderator = moderator;
        this.moderator.client.on("guildMemberAdd", async (guildMember) => {
            if (this.moderator.options.managers.mute.isMutingOnJoin) {
                const guildData = this.moderator.database.get(guildMember.guild.id);
                if (guildData) {
                    const guildMuteRole = guildData?.muteRole;
                    const guildMemberMute = guildData?.mutes?.find(guildMute => guildMute.users.target === guildMember.id && !guildMute.unmutedAt);
                    if (guildMemberMute)
                        await guildMember.roles.add(guildMuteRole);
                }
            }
        });
        this.moderator.on(ModeratorEnums_1.Events.READY, (moderator) => {
            if (moderator.options.managers.mute.isEnabled) {
                setInterval(() => {
                    const moderatorData = moderator.database.array();
                    moderatorData.forEach(guildData => {
                        const guildMutesData = guildData.mutes;
                        guildMutesData.forEach(async (guildMute) => {
                            if (!guildMute.unmutedAt && (Date.now() - guildMute.mutedAt) > guildMute.time) {
                                const guild = await this.moderator.fetcher.guild(guildData.guildID);
                                if (guild) {
                                    const unmuteTarget = await this.moderator.fetcher.member(guild.id, guildMute.users.target);
                                    await this.delete({
                                        user: unmuteTarget,
                                        guild: guild
                                    });
                                }
                            }
                        });
                    });
                }, this.moderator.options.managers.mute.interval);
            }
        });
    }
    /**
     * Method for setting the server role mute
     *
     * @param {SetMuteRoleOptions} options Set mute role options
     *
     * @returns {Promise<DefaultObject>} Returns the state of action execution
     */
    setRole(options) {
        return new Promise(async (res, rej) => {
            let guild = null;
            let guildRole = null;
            if (options.guild instanceof discord_js_1.Guild)
                guild = options.guild;
            else
                guild = await this.moderator.fetcher.guild(options.guild);
            if (!guild)
                return res({
                    status: false,
                    message: `Discord guild with ID '${options.guild}' is not found in the client!`
                });
            if (options.role instanceof discord_js_1.Role)
                guildRole = options.role;
            else
                guildRole = await this.moderator.fetcher.role(guild.id, options.role);
            if (!guildRole)
                return res({
                    status: false,
                    message: `The role with ID '${options.role instanceof discord_js_1.Role ? options.role.id : options.role}' is not found in the Discord guild with ID '${options.guild instanceof discord_js_1.Guild ? options.guild.id : options.guild}'!`
                });
            this.moderator.utils.fetchGuild(guild.id);
            const guilData = this.moderator.database.get(guild.id);
            guilData.muteRole = guildRole.id;
            this.moderator.database.set(guild.id, guilData);
            return res({ status: guilData.muteRole != null });
        });
    }
    /**
     * A method to get a server role mute
     *
     * @param {ModeratorGuild} guild Discord Guild Object or ID
     *
     * @returns {Promise<Role>} Returns server mute role or object with error
     */
    getRole(guild) {
        return new Promise(async (res, rej) => {
            let clientGuild = null;
            if (guild instanceof discord_js_1.Guild)
                clientGuild = guild;
            else
                clientGuild = await this.moderator.fetcher.guild(guild);
            if (!clientGuild)
                return res({
                    status: false,
                    message: `Discord guild with ID '${guild instanceof discord_js_1.Guild ? guild.id : guild}' is not found in the client!`
                });
            this.moderator.utils.fetchGuild(clientGuild.id);
            const guilData = this.moderator.database.get(clientGuild.id);
            const muteRole = await this.moderator.fetcher.role(clientGuild.id, guilData?.muteRole);
            return res(muteRole || null);
        });
    }
    /**
     * Method for removing the server role mute
     *
     * @param {ModeratorGuild} guild Discord Guild Object or ID
     *
     * @returns {Promise<DefaultObject>} Returns the state of action execution
     */
    deleteRole(guild) {
        return new Promise(async (res, rej) => {
            let clientGuild = null;
            if (guild instanceof discord_js_1.Guild)
                clientGuild = guild;
            else
                clientGuild = await this.moderator.fetcher.guild(guild);
            if (!clientGuild)
                return res({
                    status: false,
                    message: `Discord guild with ID '${guild instanceof discord_js_1.Guild ? guild.id : guild}' is not found in the client!`
                });
            this.moderator.utils.fetchGuild(clientGuild.id);
            const guilData = this.moderator.database.get(clientGuild.id);
            guilData.muteRole = null;
            this.moderator.database.set(clientGuild.id, guilData);
            return res({ status: guilData.muteRole === null });
        });
    }
    /**
     * Method for issuing a muth to a Discord Guild user
     *
     * @param {CreateMuteOptions} options Create mute options
     *
     * @returns {Promise<GuildMute | DefaultObject>} Returns an object with muta data or an object with an error
     */
    create(options) {
        return new Promise(async (res, rej) => {
            let guild = null;
            let member = null;
            if (options.guild instanceof discord_js_1.Guild)
                guild = options.guild;
            else
                guild = await this.moderator.fetcher.guild(options.guild);
            if (!guild)
                return res({
                    status: false,
                    message: `Discord guild with ID '${options.guild}' is not found in the client!`
                });
            if (options.user instanceof discord_js_1.GuildMember)
                member = options.user;
            else
                member = await this.moderator.fetcher.member(guild.id, options.user);
            if (!member)
                return res({
                    status: false,
                    message: `The user with ID '${options.user instanceof discord_js_1.GuildMember ? options.user.id : options.user}' is not found in the Discord guild with ID '${options.guild instanceof discord_js_1.Guild ? options.guild.id : options.guild}'!`
                });
            this.moderator.utils.fetchGuild(guild.id);
            const guilData = this.moderator.database.get(guild.id);
            const guildMemberMute = guilData?.mutes?.find(mute => mute.users.target === member.id && !mute.unmutedAt);
            const defaultMember = {
                target: member,
                moderator: (await this.moderator.fetcher.member(guild.id, options.moderator ?? this.moderator.client.user.id))
            };
            if (!guilData?.muteRole)
                return res({
                    status: false,
                    message: `To issue mutes to server users with ID '${guild.id}' you must set the role with <MuteManager>.setRole()`
                });
            if ([ModeratorEnums_1.MuteType.TEMPORARY].includes(options.type) && !options.time)
                return res({
                    status: false,
                    message: "To issue a temporary mute it is necessary to specify the time for which it will be issued!"
                });
            if (guildMemberMute)
                return res({
                    status: false,
                    message: `The user with ID '${member.id}' already has a lock on the server with ID '${guild.id}'!`
                });
            if (defaultMember.target.roles.highest.position >= defaultMember.moderator.roles.highest.position)
                return res({
                    status: false,
                    message: `The target role with ID '${defaultMember.target.id}' is above and overlaps the moderator role '${defaultMember.moderator.id}'!`
                });
            if (defaultMember.target.permissions.has("Administrator"))
                return res({
                    status: false,
                    message: `The target with ID '${defaultMember.target.id}' has administrator permissions, which does not allow to give out a mute!`
                });
            const defaultMuteData = {
                id: this.moderator.utils.generateID(),
                type: options.type,
                channelID: options.channel instanceof discord_js_1.TextChannel ? options.channel.id : options.channel,
                users: {
                    target: member.id,
                    moderator: options.moderator ?? this.moderator.client.user.id
                },
                time: options.time ?? null,
                reason: options.reason ?? null,
                mutedAt: Date.now(),
                unmutedAt: null
            };
            try {
                await member.roles.add(guilData.muteRole);
            }
            catch (error) {
                return res({
                    status: false,
                    message: error.message
                });
            }
            guilData.mutes.push(defaultMuteData);
            this.moderator.database.set(guild.id, guilData);
            this.moderator.emit(ModeratorEnums_1.Events.MUTE_STARTED, guild, defaultMuteData);
            return res(defaultMuteData);
        });
    }
    /**
     * A method for obtaining data on the user's active mute on the Discord guild
     *
     * @param {FetchOptions} options Fetch options
     *
     * @returns {Promise<GuildMute>} Returns information about a mute or an object with an error
     */
    get(options) {
        return new Promise(async (res, rej) => {
            if (!options.guild && !options.user) {
                let userMute = null;
                this.moderator.database.forEach(guildData => {
                    const guildUserMute = guildData.mutes.find(guildMute => guildMute.id === options.id);
                    if (guildUserMute?.id)
                        userMute = guildUserMute;
                });
                return res(userMute);
            }
            else {
                const guildData = this.moderator.database.get(options.guild);
                if (!guildData)
                    return res({
                        status: false,
                        message: `Discord guild with ID '${options.guild}' not found in database!`
                    });
                const guildMemberMute = guildData.mutes.find(guildMute => guildMute.id && guildMute.users.target === options.user);
                if (!guildMemberMute)
                    return res({
                        status: false,
                        message: `Mute with ID '${options.id}' was not found with user with ID '${options.user}'!`
                    });
                return res(guildMemberMute);
            }
        });
    }
    /**
     * A method for obtaining an array of mutes of a specific Discord guild or user from it
     *
     * @param {ListOptions} options Fetch options
     *
     * @returns {Promise<Array<GuildMute>>} Returns an array of Discord mutes of a guild or user
     */
    list(options) {
        return new Promise(async (res, rej) => {
            this.moderator.utils.fetchGuild(options.guild);
            const guilData = this.moderator.database.get(options.guild);
            if (options.user) {
                const guildMemberMutes = guilData?.mutes?.filter(mute => mute.users.target === options.user);
                return res(guildMemberMutes || []);
            }
            else
                return res(guilData?.mutes || []);
        });
    }
    /**
     * A method for removing muths from Discord guild users
     *
     * @param {DeleteMuteOptions} options Delete mute options
     *
     * @returns {Promise<DefaultObject>} Returns the state of the action or an object with an error
     */
    delete(options) {
        return new Promise(async (res, rej) => {
            let guild = null;
            let member = null;
            if (options.guild instanceof discord_js_1.Guild)
                guild = options.guild;
            else
                guild = await this.moderator.fetcher.guild(options.guild);
            if (!guild)
                return res({
                    status: false,
                    message: `Discord guild with ID '${options.guild}' is not found in the client!`
                });
            if (options.user instanceof discord_js_1.GuildMember)
                member = options.user;
            else
                member = await this.moderator.fetcher.member(guild.id, options.user);
            if (!member)
                return res({
                    status: false,
                    message: `The user with ID '${options.user instanceof discord_js_1.GuildMember ? options.user.id : options.user}' is not found in the Discord guild with ID '${options.guild instanceof discord_js_1.Guild ? options.guild.id : options.guild}'!`
                });
            this.moderator.utils.fetchGuild(guild.id);
            const guilData = this.moderator.database.get(guild.id);
            const guildMemberMute = guilData?.mutes?.find(mute => mute.users.target === member.id && !mute.unmutedAt);
            if (!guilData?.muteRole)
                return res({
                    status: false,
                    message: `To remove mutes from server users with ID '${guild.id}', you must set the role with <MuteManager>.setRole()`
                });
            if (!guildMemberMute)
                return res({
                    status: false,
                    message: `The target with ID '${member.id}' has no locks on the server with ID '${guild.id}'!`
                });
            try {
                await member.roles?.remove(guilData.muteRole);
            }
            catch (error) {
                return res({
                    status: false,
                    message: error.message
                });
            }
            guildMemberMute.unmutedAt = Date.now();
            guilData?.mutes?.filter(mute => mute.users.target != member.id && !mute.unmutedAt);
            guilData?.mutes?.push(guildMemberMute);
            this.moderator.database.set(guild.id, guilData);
            this.moderator.emit(ModeratorEnums_1.Events.MUTE_ENDED, guild, guildMemberMute);
            return res({ status: member.roles?.cache.has(guilData.muteRole) });
        });
    }
}
exports.MuteManager = MuteManager;
